#
#  $Id: Backup.py,v 1.10 1999/03/11 08:58:24 rob Exp $
#
#  Copyright 1998-1999 Rob Tillotson <robt@debian.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Library General Public License, version 2,
#  as published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Library General Public License for more details.
#
#  You should have received a copy of the GNU Library General Public License
#  along with this program; if not, write the Free Software Foundation,
#  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
#
"""Backup.
"""

__version__ = '$Id: Backup.py,v 1.10 1999/03/11 08:58:24 rob Exp $'

__copyright__ = 'Copyright 1998-1999 Rob Tillotson <robt@debian.org>'

import PDA.Palm
import PDA.Palm.Conduit

import os, string, stat, sys

from PDA.Palm import Options, protect_name

backup_dir = None

class Conduit(PDA.Palm.Conduit.Conduit):
    name = 'Backup'
    version = '$Version$'
    author = 'Rob Tillotson <robt@debian.org>'
    url = ''
    description = 'Backs up the PalmOS device.'
    
    def __init__(self):
	PDA.Palm.Conduit.Conduit.__init__(self)

	# options and configuration
	#self.options = ['backup_dir', 'ignore_backup_bit', 'incremental']
	self.define_options([
#	    Options.Filename('backup_dir', None, 'Backup Directory'),
	    Options.Boolean('ignore_backup_bit', 1, 'Ignore Backup Bit?'),
	    Options.Boolean('incremental', 1, 'Incremental?'),
	    # temporary
	    Options.Boolean('debug', 0, 'Debugging Output'),
	    ])

    def __call__(self, dlp, username):
	path = os.path.join(self.registry.user_directory(username), 'backup')
	
	self.log('user: "%s"  path: "%s"' % (username, path))
	try:
	    os.stat(path)
	except os.error:
	    os.mkdir(path)

	dblist = os.listdir(path)  # for use in incremental backup
	
	i = 0
	while 1:
	    info = dlp.getDBInfo(i, 1, 0, 0)
	    if not info: break
	    i = info['index'] + 1

	    if self.ignore_backup_bit or info['flagBackup']:
		if info['flagResource']: fn = info['name'] + '.prc'
		else: fn = info['name'] + '.pdb'
		fn = protect_name(fn)
		fp = os.path.join(path, fn)
		
		info['flagOpen'] = 0

		doit = 0
		if not self.incremental or fn not in dblist:
		    doit = 1
		else:
		    fi = PDA.Palm.openFile(fp, None).info
		    if self.debug:
			self.log('%s %s/%s : %s/%s %s' % (fn, fi['modifyDate'],
							  fi['modnum'],
							  info['modifyDate'],
							  info['modnum'],
							  info['flagBackup']))

		    # the BackupBuddy Rule:
		    #  Under normal circumstances, tweaking a database should
		    #  cause BOTH modnum and modifyDate to be changed at the
		    #  same time.  Thus:
		    #
		    #  - if the modnum differs but modifyDate is the same,
		    #    then the DB has been touched by a BB-certified app.
		    #  - if the modnum is the same but modifyDate is different,
		    #    the DB has not been touched by the app, but has been
		    #    moved by the OS (which it tends to do to ALL OF THEM
		    #    AT ONCE :).
		    #

		    if (fi['modnum'] != info['modnum'] and \
		       fi['modifyDate'] != info['modifyDate']) or \
		       fi['createDate'] != info['createDate']:
			doit = 1
			
		if doit:
		    self.log('backing up "%s"' % info['name'])
		    f = PDA.Palm.createFile(fp, info, None)
		    try:
			f.retrieve(dlp)
			f.close()
		    except:
			self.log('failed to back up "%s"' % info['name'])
			f.close()
			try:
			    os.unlink(fp)
			except:
			    pass
		    
	    if fn in dblist: dblist.remove(fn)
	    
	if self.incremental:
	    for fn in dblist:
		self.log('removing "%s" from backup directory' % fn)
		os.unlink(os.path.join(path, fn))
		
	self.log('done.')

