#
#  $Id: DBTypes.py,v 1.1 1999/02/02 02:32:55 rob Exp $
#
#  Copyright 1998-1999 Rob Tillotson <robt@debian.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Library General Public License, version 2,
#  as published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Library General Public License for more details.
#
#  You should have received a copy of the GNU Library General Public License
#  along with this program; if not, write the Free Software Foundation,
#  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
#
"""Database type/class registry.

This module maintains a set of associations between PalmOS database
information and the classes that implement each type.  Application
modules can, when loaded, register types with this module for
automatic typing of opened files.
"""

__version__ = '$Id: DBTypes.py,v 1.1 1999/02/02 02:32:55 rob Exp $'

__copyright__ = 'Copyright 1998-1999 Rob Tillotson <robt@debian.org>'


import re, operator

class DictAssoc:
    """An instance of this class maintains a list of dictionary->object
    associations.  When called with a dictionary parameter, it matches
    the parameter against each dictionary in the list in turn, returning
    the associated object when it finds a match.  The semantics of
    of the comparison are:
      - excess items in the parameter are ignored
      - string values in the association are treated as regular
        expressions to be matched against strings in the parameter
      - everything else is simply compared
      - all fields in the association must match in order for the
        association to be considered a match
    """
    def __init__(self, default):
	self.data = []
	self.default = default

    def register(self, cls, match):
	if (match, cls) not in self.data: self.data.append((match, cls))

    def unregister(self, cls, match):
	if (match, cls) in self.data: self.data.remove((match, cls))

    def __call__(self, target):
	for match, cls in self.data:
	    m = match.keys()
	    # Who says Python is always easy to read? :-)
	    if not filter(lambda x: not x,
		      map(lambda x,y: type(x) == type('') and \
			  re.match(x,y) or x == y,
			  map(operator.__getitem__, [match]*len(m), m),
			  map(lambda x,y: x.get(y,''), [target]*len(m), m))):
		return cls
	return self.default
    

# Initialization
def registry_defaults(db, pref):
    Types.default = db
    PrefTypes.default = pref

if not globals().has_key('Types'): Types = DictAssoc(None)
if not globals().has_key('PrefTypes'): PrefTypes = DictAssoc(None)


