#
#  $Id: Plugin.py,v 1.4 1999/03/11 08:58:16 rob Exp $
#
#  Copyright 1998-1999 Rob Tillotson <robt@debian.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Library General Public License, version 2,
#  as published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Library General Public License for more details.
#
#  You should have received a copy of the GNU Library General Public License
#  along with this program; if not, write the Free Software Foundation,
#  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
#
"""Plug-in architecture.

  This module defines a standard 'plug in' architecture.  Most of the
  code here used to be in the Conduit class, but was moved here so that
  it would be useful for other sorts of plug-ins, notably import/export
  filters.
"""

__version__ = '$Id: Plugin.py,v 1.4 1999/03/11 08:58:16 rob Exp $'

__copyright__ = 'Copyright 1999 Rob Tillotson <robt@debian.org>'

import sys, os, string, imp
from types import ModuleType, ClassType
# fix this
import Config
#
import Options
#


class Plugin:
    """
    Superclass for all plug-ins.
    """
    name = ''
    author = ''
    description = ''
    version = ''
    url = ''
    type = ''
    properties = []

    # needed by find_plugins
    is_plugin = 1
    
    def __init__(self):
	self.logfile = sys.stdout
	
	self.options = {}
	self.registry = None

    def __repr__(self):
	return '<Plugin: %s: %s>' % (self.type, self.name)

    __str__ = __repr__
    
    def log(self, str):
	"""Write a string to the log file.
	"""
	self.logfile.write('%s: %s\n' % (self.name, str))

    def define_options(self, options):
	"""Define plug-in options; for internal use.
	"""
	for x in options:
	    if isinstance(x, Options.Option):
		self.options[x.name] = x

    #
    # Overload __setattr__ and __getattr__ to transparently treat options
    # like instance attributes
    #
    def __getattr__(self, k):
	if self.__dict__.has_key('options') and self.options.has_key(k):
	    return self.options[k].value

    def __setattr__(self, k, v):
	if self.__dict__.has_key('options') and self.__dict__['options'].has_key(k):
	    self.__dict__['options'][k].validate(v)
	    self.__dict__['options'][k].value = v
	else:
	    self.__dict__[k] = v
	    
    def gui_configure(self, parent, *a, **kw):
	"""Pop up a GUI configuration dialog.  Right now this just uses
	the Tk one, but later there will be a way to select a method.
	"""
	# there should also be a way to specify the order things appear
	# in the dialog.
	from PDA.Palm.Tk import ConfigDialog
	apply(ConfigDialog.ConfigDialog, (parent,self.options.values())+a, kw)
	    

    def configure(self, options=None):
	"""Set plug-in options.  The optional parameter is either a
	dictionary (containing option values, keyed by name) or a
	Registry object; if either is supplied, the conduit will be
	configured from that data.  If no parameter is supplied, the
	current option values will be returned as a dictionary.
	"""
	if options is None:
	    d = {}
	    for k in self.options.values():
		d[k.name] = k.value
	elif isinstance(options, Config.Registry):
	    self.registry = options
	    for k in self.options.values():
		if self.registry.has_key('Pyrite',
					 '%s.%s.%s' % (self.type, self.name, k.name)):
		    v = options.get('Pyrite',
				    '%s.%s.%s' % (self.type, self.name, k.name))
		    try:
			k.from_str(v)
		    except Options.ValidationError:
			k.value = k.default
		# backward compatibility below
		elif self.registry.has_key('PalmPython',
					 '%s.%s.%s' % (self.type, self.name, k.name)):
		    v = options.get('PalmPython',
				    '%s.%s.%s' % (self.type, self.name, k.name))
		    try:
			k.from_str(v)
		    except Options.ValidationError:
			k.value = k.default
		else:
		    k.value = k.default
	else:
	    for k in self.options.values():
		if options.has_key(k.name):
		    try:
			k.validate(options[k.name])
		    except Options.ValidationError:
			k.value = k.default
		    else:
			k.value = options[k.name]
		else:
		    k.value = k.default


    def has_property(self, prop):
	"""Test whether this plug-in has the named property.
	"""
	return prop in self.properties
    

def list_modules(path):
    """List all available modules in a given directory."""
    l = {}
    suff = map(lambda x: x[0], imp.get_suffixes())
    suff.sort(lambda x,y: cmp(y,x))
    for name in os.listdir(path):
	for s in suff:
	    if name[-len(s):] == s:
		l[name[:-len(s)]] = 1
		break

    return l.keys()


def find_plugins(package):
    """Find all plugins in a given package.

    This function is used to enumerate all the available plug-ins in
    a subpackage.  It assumes that a package of plug-ins is
    organized as a flat set of submodules, each of which contains
    one or more plug-ins.  At this time, that is how Pyrite's
    plug-ins are organized; if they are reorganized into a more complex
    structure later, this function will have to be rewritten.

    Plug-ins are identified by searching each submodule for class
    objects, and then checking each of those objects for the 'is_plugin'
    attribute.  This is a compromise, since there doesn't seem to be a
    better way of doing it.  The informational attributes of the
    class are then copied to a dictionary.

    The return value of this function is a list of 3-tuples, containing
    the name of the submodule, the name of the plug-in class, and a
    dictionary of informational attributes.

    As a side effect, this function imports the given package and all
    submodules, but it doesn't pollute any namespaces (except sys.modules).
    It also does not instantiate any plug-in objects.
    """
    plugins = []
    ns = {}
    exec 'from %s import *' % package in ns

    modules = filter(lambda x: type(x[1]) == ModuleType, ns.items())
    for mn, m in modules:
	for cn, c in filter(lambda x: type(x[1]) == ClassType, m.__dict__.items()):
	    if hasattr(c, 'is_plugin'):
		plugins.append( (mn, cn, {'name': c.name,
					  'type': c.type,
					  'description': c.description,
					  'version': c.version,
					  'url': c.url,
					  'properties': c.properties
					  }) )
    return plugins
