#
#  $Id: DLP.py,v 1.2 1999/03/11 08:58:24 rob Exp $
#
#  Copyright 1999 Rob Tillotson <robt@debian.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Library General Public License, version 2,
#  as published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Library General Public License for more details.
#
#  You should have received a copy of the GNU Library General Public License
#  along with this program; if not, write the Free Software Foundation,
#  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
#
"""Databases on an attached Palm Computing(R) platform device.
"""

__version__ = '$Id: DLP.py,v 1.2 1999/03/11 08:58:24 rob Exp $'

__copyright__ = 'Copyright 1999 Rob Tillotson <robt@debian.org>'

import PDA.Palm.Store
import PDA.Palm
from PDA.Palm import _pdapalm, Database

class Store(PDA.Palm.Store.Store):
    name = 'DLP'
    version = '$Version$'
    author = 'Rob Tillotson <robt@debian.org>'
    description = 'Contents of an attached device.'
    properties = ['read','write','create','delete']
    
    def __init__(self, port='/dev/pilot', cardno=0):
	PDA.Palm.Store.Store.__init__(self)

	self.__socket = _pdapalm.openPort(port)
	self.dlp = _pdapalm.accept(self.__socket)
	# set the message on the display
	self.dlp.status()
	self.cardno = cardno

    def open(self, name, mode='rws', dbclass=Database):
	m = 0
	if 'r' in mode: m = m | _pdapalm.OpenDBRead
	if 'w' in mode: m = m | _pdapalm.OpenDBWrite
	if 'x' in mode: m = m | _pdapalm.OpenDBExclusive
	if 's' in mode: m = m | _pdapalm.OpenDBSecret
	i = self.dlp.findDBInfo(0, name, None, None, self.cardno)
	db = self.dlp.open(name, m, self.cardno)
	return dbclass(db, i)

    def create(self, name, creator, type, flags=0, version=1,
	       dbclass=Database):
	db = self.dlp.create(name, creator, type, flags, version, self.cardno)
	i = self.dlp.findDBInfo(0, name, None, None, self.cardno)
	return dbclass(db, i)

    def delete(self, name):
	return self.dlp.delete(name, self.cardno)

    def info(self, name):
	return self.dlp.findDBInfo(0, name, None, None, self.cardno)

    def list(self):
	return map(lambda x: x['name'], self.listinfo())
    
    def listinfo(self, name=None, creator=None, type=None):
	l = []
	i = 0
	while 1:
	    info = self.dlp.getDBInfo(i, 1, 0, self.cardno)
	    if not info: break
	    i = info['index']+1
	    if name is not None and info['name'] != name: continue
	    if creator is not None and info['creator'] != creator: continue
	    if type is not None and info['type'] != type: continue
	    l.append(info)
	return l
    

    
