#
#  $Id: Directory.py,v 1.1 1999/02/02 02:33:05 rob Exp $
#
#  Copyright 1999 Rob Tillotson <robt@debian.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Library General Public License, version 2,
#  as published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Library General Public License for more details.
#
#  You should have received a copy of the GNU Library General Public License
#  along with this program; if not, write the Free Software Foundation,
#  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
#
"""Store in a local directory.
"""

__version__ = '$Id: Directory.py,v 1.1 1999/02/02 02:33:05 rob Exp $'

__copyright__ = 'Copyright 1999 Rob Tillotson <robt@debian.org>'

import os

import PDA.Palm.Store
from PDA.Palm import pdbfile, protect_name, Database
from PDA.Palm import _pdapalm


class Store(PDA.Palm.Store.Store):
    name = 'Directory'
    author = 'Rob Tillotson <robt@debian.org>'
    version = '$Version$'
    description = 'PRC/PDB files in a directory.'
    properties = ['read']
    
    def __init__(self, path='.'):
	PDA.Palm.Store.Store.__init__(self)

	self.path = path

    def open(self, name, mode='rws', dbclass=Database):
	# note: silently opens all files for read.
	db = self._find_db(name, dbclass)
	if not db: raise IOError, 'no such database: %s' % name
	return db

    def create(self, name, creator, type, flags=0, version=1, dbclass=Database):
	raise RuntimeError, 'unimplemented'

    def delete(self, name):
	raise RuntimeError, 'unimplemented'

    def info(self, name):
	db = self._find_db(name)
	if not db: raise IOError, 'no such database: %s' % name
	return db.info

    def list(self):
	return map(lambda x: x[1]['name'], pdbfile.listdir(self.path))

    def listinfo(self, name=None, creator=None, type=None):
	l = []
	for i in map(lambda x: x[1], pdbfile.listdir(self.path)):
	    if name is not None and i['name'] != name: continue
	    if creator is not None and i['creator'] != creator: continue
	    if type is not None and i['type'] != type: continue
	    l.append(i)

	return l	    

    def _try_open_db(self, fname, dbname=None, dbclass=Database):
	try:
	    f = _pdapalm.fileOpen(fname)
	    i = f.getDBInfo()

	    if dbname is None or i['name'] == dbname: return dbclass(f, i)
	    else: return None
	except:
	    return None
	
    def _find_db(self, name, dbclass=Database):
	# dbname.pdb
	db = self._try_open_db(os.path.join(self.path,protect_name(name)+'.pdb'),
			       name, dbclass)
	# dbname.prc
	if not db: db = self._try_open_db(os.path.join(self.path,
						      protect_name(name)+'.prc'),
					 name, dbclass)
	# basic name
	if not db: db = self._try_open_db(os.path.join(self.path,protect_name(name)),
					 name, dbclass)
	# any arbitrary file
	for n, i in pdbfile.listdir(self.path):
	    if i['name'] == name:
		db = self._try_open_db(os.path.join(self.path,n), name, dbclass)
		break

	return db
    
