#!/usr/bin/python
#
#  $Id: pdbdiff.py,v 1.1 1999/02/02 02:33:12 rob Exp $
#
#  Copyright 1999 Rob Tillotson <robt@debian.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Library General Public License, version 2,
#  as published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Library General Public License for more details.
#
#  You should have received a copy of the GNU Library General Public License
#  along with this program; if not, write the Free Software Foundation,
#  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
#
"""Compare two databases.

  This utility describes the differences between two databases.  It is
  meant to be a debugging aid, so it attempts to be rather clueful
  about database structure, reporting exactly what differs in some
  detail.

  The major weakness of this program at the moment is that it compares
  records in order of appearance, not by ID; that might be added as an
  option later.
"""

__version__ = '$Id: pdbdiff.py,v 1.1 1999/02/02 02:33:12 rob Exp $'

__copyright__ = 'Copyright 1998-1999 Rob Tillotson <robt@debian.org>'

import sys, os, string
import PDA.Palm

info_fields = [
    'name', 'type', 'creator', 'createDate', 'modifyDate', 'backupDate',
    'modnum', 'version', 'flagReset', 'flagResource', 'flagNewer',
    'flagExcludeFromSync', 'flagAppInfoDirty', 'flagReadOnly', 'flagBackup',
    'flagOpen', 'more', 'index'
    ]

def diff_info(i0, i1):
    """Compare the two passed info structures.  Ignores fields not listed in
    info_fields."""
    for f in info_fields:
	if not i0.has_key(f):
	    print "0: missing info field '%s'" % f
	if not i1.has_key(f):
	    print "1: missing info field '%s'" % f
	if i0.has_key(f) and i1.has_key(f):
	    if i0[f] != i1[f]:
		print "field '%s' differs: '%s' / '%s'" % (f, i0[f], i1[f])

def diff_appinfo(db0, db1):
    a0 = db0.getAppBlock()
    a1 = db1.getAppBlock()

    if a0.raw is None and a1.raw is not None:
	print "db0: has no appinfo block"
    elif a0.raw is not None and a1.raw is None:
	print "db1: has no appinfo block"
    elif a0.raw is not None and a1.raw is not None:
	if len(a0.raw) != len(a1.raw):
	    print "appinfo lengths differ: %d / %d" % (len(a0.raw), len(a1.raw))
	elif a0.raw != a1.raw:
	    print "appinfo contents differ"

def diff_sortinfo(db0, db1):
    a0 = db0.getSortBlock()
    a1 = db1.getSortBlock()

    if a0.raw is None and a1.raw is not None:
	print "db0: has no sortinfo block"
    elif a0.raw is not None and a1.raw is None:
	print "db1: has no sortinfo block"
    elif a0.raw is not None and a1.raw is not None:
	if len(a0.raw) != len(a1.raw):
	    print "sortinfo lengths differ: %d / %d" % (len(a0.raw), len(a1.raw))
	elif a0.raw != a1.raw:
	    print "sortinfo contents differ"

rec_attrs = [ 'id', 'category', 'deleted', 'modified', 'busy', 'secret', 'archived' ]

def diff_records(db0, db1):
    rc0 = db0.getRecords()
    rc1 = db1.getRecords()

    if rc0 != rc1:
	print "record counts differ: %d / %d" % (rc0, rc1)

    c = min(rc0, rc1)
    for x in range(0, c):
	r0 = db0.getRecord(x)
	r1 = db1.getRecord(x)

	for a in rec_attrs:
	    a0 = getattr(r0, a)
	    a1 = getattr(r1, a)
	    if a0 != a1:
		print "record #%s: %s: %s / %s" % (x, a, a0, a1)

	if r0.raw == None: raw0 = ''
	else: raw0 = r0.raw
	if r1.raw == None: raw1 = ''
	else: raw1 = r1.raw
	
	if len(raw0) != len(raw1):
	    print "record #%s: length: %s / %s" % (x, len(raw0), len(raw1))
	elif raw0 != raw1:
	    print "record #%s: contents" % x
	    

def diff(db0, db1):
    """diff two open databases."""

    diff_info(db0.info, db1.info)

    diff_appinfo(db0, db1)
    diff_sortinfo(db0, db1)
    diff_records(db0, db1)
    

def main(argv):
    if len(argv) < 2:
	print "Need to specify at least two files."
	return None

    f0 = PDA.Palm.openFile(argv[0])
    f1 = PDA.Palm.openFile(argv[1])

    diff(f0, f1)

if __name__ == '__main__':
    main(sys.argv[1:])
    
